/*
Volcan - algorithme de simulation pour SimuApplet

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;
import java.util.*;

/**
 * Algorithme de simulation pour SimuApplet.
 * Simule un projectile lanc sur Terre avec une vitesse initiale donne et un angle
 * donn.
 */
public class Volcan extends SimuApplet {

    static final double g = 0.00180;  // en km !!
    double dt = 10;
    double entre2 = 0.01;
    double x,y,z,r,eta;
    double vr,vz,azim,angleVisee;
    int nbtraj=0,nbPartic=20;
    
    // constructeur de la classe, avec la dfinition des paramtres de simulation
    // et des options d'affichage.
    public Volcan() {
        // paramtres d'entre
        ParamIn v0 = new ParamIn();
        v0.type = "nombre";
        v0.label = "v0";
        v0.titre = "Vitesse d'jection";
        v0.unit = "km/s";
        v0.acquisition = "champ";
        v0.defaut = "0.5";
        addParamIn(v0);
        
        ParamIn ouvert = new ParamIn();
        ouvert.type = "nombre";
        ouvert.label = "demiAngleOuverture";
        ouvert.titre = "Demi-angle d'ouverture du cne d'jection";
        ouvert.unit = "degrs";
        ouvert.acquisition = "champ";
        ouvert.defaut = "55";
        addParamIn(ouvert);
        
        ParamIn visee = new ParamIn();
        visee.type = "nombre";
        visee.label = "angleDeVisee";
        visee.titre = "Angle de vise par rapport  la verticale";
        visee.unit = "degrs";
        visee.acquisition = "champ";
        visee.defaut = "0";
        addParamIn(visee);
        
        ParamIn nbPartic = new ParamIn();
        nbPartic.type = "nombre";
        nbPartic.label = "nbPartic";
        nbPartic.titre = "Nombre de trajectoires";
        nbPartic.acquisition = "champ";
        nbPartic.defaut = "20";
        addParamIn(nbPartic);
        
        // paramtres de sortie
        ParamOut xv = new ParamOut();
        xv.type = "nombre";
        xv.label = "x";
        addParamOut(xv);
        ParamOut yv = new ParamOut();
        yv.type = "nombre";
        yv.label = "y";
        addParamOut(yv);
        ParamOut etav = new ParamOut();
        etav.type = "nombre";
        etav.label = "eta";
        addParamOut(etav);
        ParamOut zv = new ParamOut();
        zv.type = "nombre";
        zv.label = "z";
        addParamOut(zv);
        
        // options d'affichage
        // affichage "multiplot"
        Affichage multi = new Affichage();
        multi.type = "multiplot";
        multi.titre = "Provoquer une ruption";
        multi.sousAff = new Affichage[2];
        
        Affichage m1 = new Affichage();
        m1.type = "plotlive";
        m1.titre = "Vue de ct";
        m1.xdebut = "-200";
        m1.xfin = "200";
        m1.ydebut = "0";
        m1.yfin = "100";
        m1.couleurs = false;
        String[][] pparamsm1 = {{"x", "z"}};
        m1.params = pparamsm1;
        multi.sousAff[0] = m1;
        
        Affichage m2 = new Affichage();
        m2.type = "plotlive";
        m2.titre = "Vue quelconque";
        m2.xdebut = "-200";
        m2.xfin = "200";
        m2.ydebut = "-200";
        m2.yfin = "200";
        m2.couleurs = false;
        String[][] pparamsm2 = {{"y", "eta"}};
        m2.params = pparamsm2;
        multi.sousAff[1] = m2;
        
        addAffichage(multi);
        appletDoc("volcanodoc.xml");
  }
    
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
  public void initCalculLive(ListeValeurs in) throws SimuException {
    x = 0;
    y = 0;
    z = 0;
    r = 0;
    double v0 = in.lireDouble("v0");  // en km !!
    double angleCone = in.lireDouble("demiAngleOuverture");
    angleCone = Math.PI/180*angleCone;
    angleVisee = in.lireDouble("angleDeVisee");
    angleVisee = Math.PI/180*angleVisee;
    nbPartic = (int)in.lireDouble("nbPartic");
    double rmax = v0*v0/g/2;   // en km

// pour ajuster l'chelles des ordonnes en fonction des valeurs du modle de dpart
    Vector affichages = getAffichage();
    Affichage sous_aff[] = new Affichage[2];
    for ( int k=0 ; k < affichages.size() ; k++ ) {
      for ( int l=0 ; l < ((Affichage)affichages.elementAt(k)).sousAff.length ; l++ ) {
        sous_aff[l] = ((Affichage)affichages.elementAt(k)).sousAff[l];
        if ( sous_aff[l].titre == "Vue de ct" ) {
          sous_aff[l].xdebut = Double.toString(-2*rmax);
          sous_aff[l].xfin = Double.toString(2*rmax);
          sous_aff[l].yfin = Double.toString(rmax);
        }
        if ( sous_aff[l].titre == "Vue quelconque" ) {
          sous_aff[l].xdebut = Double.toString(-2*rmax);
          sous_aff[l].xfin = Double.toString(2*rmax);
          sous_aff[l].ydebut = Double.toString(-2*rmax*Math.cos(angleVisee));
          sous_aff[l].yfin = Double.toString(2*rmax*Math.cos(angleVisee)+rmax*Math.sin(angleVisee));
        }
      }
    }

    double cmax = Math.cos(angleCone);

    double a1 = Math.random();
    double a2 = Math.random();
    double a3 = Math.random();
    
    double c = (cmax-1)*a1+1.;
    double angle = Math.acos(c);
    
    azim = 2*Math.PI*a3;

    vr = v0*Math.cos(angle);
    vz = v0*Math.sin(angle);
  }
    
    // Calcul du temps t au temps t+1.
    public ListeValeurs calculLive() throws SimuException {
        ListeValeurs out = new ListeValeurs();
        if (z < 0) {
            nbtraj++;
//            System.out.println("nbtraj"+nbtraj);
//            System.out.println("nbPartic"+nbPartic);
            if (nbtraj >= nbPartic) {
                nbtraj = 0;
                out.ajouterString("fin", "simulation");
            } else
                out.ajouterString("fin", "trajectoire");
            return(out);
        }
        r += vr * dt;
        x = r*Math.cos(azim);
        y = r*Math.sin(azim);
        z += vz * dt;
        eta = x*Math.cos(angleVisee) + z*Math.sin(angleVisee);
        vz -= g * dt;
        
        out.ajouterDouble("x", x);
        out.ajouterDouble("y", y);
        out.ajouterDouble("z", z);
        out.ajouterDouble("eta", eta);
        dormir(Math.round(entre2*1000)); // pour bloquer le programme pendant entre2 secondes
        return out;
    }
    
    // calcul complet d'un coup (utilis avec les affichages "plot" et "tableau")
    public ListeValeurs calcul(ListeValeurs in) throws SimuException {
/*        x = 0;
        y = 0;
        double v0 = in.lireDouble("v0");
        double angle = in.lireDouble("direction");
        vx = v0 * Math.cos(angle/180*Math.PI);
        vy = v0 * Math.sin(angle/180*Math.PI);
        ListeValeurs out = new ListeValeurs();
        while (y >= 0) {
            x += vx * dt;
            y += vy * dt;
            vy -= g * dt;
            out.ajouterDouble("x", x);
            out.ajouterDouble("y", y);
        }
        return out;*/
        return null;
    }
}
